@@ -1,27 +1,26 @@
 BUILD_BIT='64bit'
 BUILD_CPU='Intel(R) Core(TM) i7-7700HQ CPU @ 2.80GHz'
-BUILD_DATE='2021-11-08'
+BUILD_DATE='2022-06-21'
 BUILD_MACHINE='x86_64'
 BUILD_OS='linux'
-BUILD_TIME='17:20'
+BUILD_TIME='13:48'
 BUILT_BY='antoine'
 BUILT_ON='localhost.localdomain'
-COMPILER_VERSION='gcc (GCC) 11.1.0'
-CYTHON_VERSION='0.29.24'
-LINKER_VERSION='GNU ld (GNU Binutils) 2.36.1'
-PYTHON_VERSION='3.9.7'
-RELEASE_BUILD=True
+COMPILER_VERSION='gcc (GCC) 12.1.0'
+CYTHON_VERSION='0.29.30'
+LINKER_VERSION='GNU ld (GNU Binutils) 2.38'
+PYTHON_VERSION='3.10.5'
 lib_gobject_introspection='1.68.0'
-lib_gtk='3.24.30'
-lib_gtk_x11='3.24.30'
+lib_gtk='3.24.34'
+lib_gtk_x11='3.24.34'
 lib_nvenc='10'
 lib_nvfbc='6.0'
-lib_py3cairo='1.20.1'
-lib_pygobject='3.42.0'
-lib_python3='3.9'
+lib_py3cairo='1.21.0'
+lib_pygobject='3.42.1'
+lib_python3='3.10'
 lib_vpx='1.11.0'
-lib_x11='1.7.2'
-lib_x264='0.163.3060'
+lib_x11='1.8.1'
+lib_x264='0.164.3081'
 lib_x265='3.5'
 lib_xcomposite='0.4.5'
 lib_xdamage='1.1.5'
